const Tars = require('@tars/rpc');
const TarsServer = Tars.server;
const { Endpoint } = require('@tars/utils');

class App {
  async main({ impMap, configPath }) {
    process.env.TARS_CONFIG = process.env.TARS_CONFIG || configPath;
    require('@tars/logs').setConfig(process.env.TARS_CONFIG);

    const logger = require('./DayLog')('App');
    logger.info(`process.env.TARS_CONFIG: ${process.env.TARS_CONFIG}`);

    const servantConfigs = TarsServer.getServant(process.env.TARS_CONFIG);
    logger.info(`servantConfigs: ${JSON.stringify(servantConfigs)}`);

    for (let i = 0; i < servantConfigs.length; i++) {
      const servantConfig = servantConfigs[i];
      const svr = TarsServer.createServer(impMap[servantConfig.servant]);
      svr.start(servantConfig);

      logger.info(`server ${servantConfig.servant} run in ${servantConfig.endpoint}`);
      // 从servantConfig.endpoint中解析出ip:port
      const endpoint = Endpoint.parse(servantConfig.endpoint);

      if (!process.env.TAF_CONFIG) {
        require('tx-report-servant').updateServant(servantConfig.servant, endpoint.iPort);
      }
    }
  }
}

module.exports = App;
