const Base = require('../class/Base');

class Cache extends Base {
  constructor({ interval, name } = { interval: 30 * 60 * 1000, name: '' }) {
    super({
      logName: name,
    });

    this.timer = null;
    this.interval = interval;

    process.on('message', (obj) => {
      if (/^reload_cache/.test(obj.cmd)) {
        const matched = obj.cmd.match(/^reload_cache:(.*)/);
        if (matched && matched[1] === name) {
          this.logger.info(`cache ${name} this.myselfUpdate();`);
          this.myselfUpdate();
        }
      }
    });
    setImmediate(() => {
      this.myselfUpdate();
    });
  }

  async myselfUpdate() {
    if (this.timer) {
      clearTimeout(this.timer);
      this.timer = null;
    }
    try {
      if (this.update) {
        await this.update();
      }
    } catch (e) {
      this.logger.error(`cache:${this.name} update error | ${e.stack}`);
    } finally {
      this.timer = setTimeout(this.myselfUpdate.bind(this), this.interval);
    }
  }
}

module.exports = Cache;
