const Base = require('./Base');
const axios = require('axios').default;

class CoWXRobot extends Base {
  /**
     * constructor
    *
    * @param {string} webHook 企业微信群机器人的 Webhook 地址
    */
  constructor(webHook) {
    super();

    this.webHook = webHook;
  }

  /**
    * 发送文本消息
    *
    * @param {string} content 消息内容
    * @param {string[]} mentionedList 提醒人的英文名数组
    * @param {string[]} mentionedMobileList 提醒人的手机号数组
    */
  async sendText(content, mentionedList = [], mentionedMobileList = []) {
    return await this.request({
      msgtype: 'text',
      text: {
        content,
        mentioned_list: mentionedList,
        mentioned_mobile_list: mentionedMobileList,
      },
    });
  }

  async request(req) {
    const httpResult = await axios.request({
      method: 'post',
      url: this.webHook,
      headers: { 'Content-Type': 'application/json' },
      data: JSON.stringify(req),
    });

    return httpResult;
  }
}

module.exports = CoWXRobot;
