const Base = require('./Base');

class Config extends Base {
  constructor(fileList, localBasePath = '') {
    super();

    this.fileList = fileList || [];

    this.hasInit = false;

    this.loader = null;
    if (process.env.DOCKER_ENV) {
      const tarsConf = require('../utils/tars-config-loader');
      this.loader = tarsConf;
    } else {
      const fs = require('fs');
      const path = require('path');
      const Hjson = require('hjson');

      const localConf = {
        async add(filename) {
          let name = filename.slice(0, filename.lastIndexOf('.'));
          name = name.replace('-', '_');
          const text = fs.readFileSync(path.join(localBasePath, filename), 'utf8');
          this.configs[name] = Hjson.parse(text);
        },
        configs: {},
      };

      this.loader = localConf;
    }

    this.configs = {};
  }

  async init() {
    if (this.hasInit) {
      return;
    }

    await Promise.all(this.fileList.map(file => this.loader.add(file)));

    this.configs = this.loader.configs;

    this.hasInit = true;
  }
}

module.exports = Config;
