const Base = require('./Base');
const { MongoClient, ObjectId } = require('mongodb');

module.exports = class MongoDB extends Base {
  constructor({ logName, url, mongoConf = {} }) {
    super({ logName: logName || '' });
    this.pool = null;
    this.url = url;
    this.mongoConf = mongoConf;
  }

  async init() {
    if (!this.pool) {
      this.pool = await MongoClient.connect(this.url, Object.assign({ useUnifiedTopology: true }, this.mongoConf));
    }
  }

  static ObjectId(...args) {
    return ObjectId(...args);
  }

  async getDB({ dbName }) {
    await this.init();
    return await this.pool.db(dbName);
  }

  async getCollection({ dbName, collectionName }) {
    await this.init();
    const db = await this.pool.db(dbName);
    const collection = await db.collection(collectionName);
    return collection;
  }
};
