const Base = require('../class/Base');

class TafClient extends Base {
  constructor({ jceProxy, proxyString, set, timeout, accessLog = true, resultLog = false }) {
    super();

    this.accessLog = accessLog;
    this.resultLog = resultLog;

    const { Communicator } = require('@tars/rpc');
    const client = Communicator.New();
    if (timeout) {
      client.setProperty('timeout', timeout);
    }

    if (set) {
      this.proxy = client.stringToProxy(jceProxy, proxyString, set);
    } else {
      this.proxy = client.stringToProxy(jceProxy, proxyString);
    }
  }

  checkTafRsp() { // 默认的taf返回异常处理函数
    return;
  }

  async callTaf({ funcName, reqList, context }) {
    try {
      if (this.accessLog) {
        this.logger.info(`${funcName}|ACCESS|CONTEXT: ${JSON.stringify(context)}|REQLIST: ${JSON.stringify(reqList.map(e => (e.toObject ? e.toObject() : e)))}`);
      }
      const tafRsp = await this.proxy[funcName](...reqList, { context });

      this.checkTafRsp(tafRsp);

      if (this.resultLog) {
        this.logger.info(`${funcName}|RESULT|CONETXT: ${JSON.stringify(context)}|REQLIST: ${JSON.stringify(reqList.map(e => (e.toObject ? e.toObject() : e)))}|RSP: ${JSON.stringify(Object.fromEntries(Object.entries(tafRsp.response.arguments).map(([key, value]) => [key, value.toObject ? value.toObject() : value])))}`);
      }

      return tafRsp.response.arguments;
    } catch (e) {
      this.logger.error(`${funcName}|ERROR|CONTEXT: ${JSON.stringify(context)}|STACK: ${e.stack}`);
      throw e;
    }
  }
}

module.exports = TafClient;
