const Base = require('./Base');
const axios = require('axios').default;

class TrpcClient extends Base {
  constructor({ trpcProxyHost, serviceName, timeout, accessLog = true, resultLog = false }) {
    super();

    this.trpcProxyHost = trpcProxyHost;
    this.serviceName = serviceName;
    this.accessLog = accessLog;
    this.resultLog = resultLog;
    this.timeout = timeout;
  }

  buildTransInfo(context) {
    if (!context || typeof context !== 'object') {
      return '';
    }
    const keys = Object.keys(context);
    const transInfo = {};
    for (let i = 0; i < keys.length; i++) {
      const key = keys[i];
      const value = context[key];
      transInfo[key] = Buffer.from(value).toString('base64');
    }
    if (Object.keys(transInfo).length > 0) {
      return JSON.stringify(transInfo);
    }
    return '';
  }

  checkTrpcRsp(httpResult) { // 默认的返回异常处理函数
    if (httpResult.status !== 200) {
      throw new Error(`httpResult.status!==200|httpResult.status=${httpResult.status}|httpResult.statusText=${httpResult.statusText}`);
    }
    if (httpResult.data === undefined) {
      throw new Error('httpResult.data===undefined');
    }
    return;
  }

  async callTrpc({ funcName, req, context }) {
    try {
      if (this.accessLog) {
        this.logger.info(`${funcName}｜ACCESS|CONTEXT: ${JSON.stringify(context)}|REQ: ${JSON.stringify(req.toObject ? req.toObject() : req)}`);
      }
      const transInfo = this.buildTransInfo(context);
      const headers = { 'Content-Type': 'application/json' };
      if (transInfo) {
        headers['trpc-trans-info'] = transInfo;
      }

      const httpResult = await axios.request({
        method: 'post',
        url: `${this.trpcProxyHost}/${this.serviceName}/${funcName}`,
        headers,
        data: JSON.stringify(req),
      });

      this.checkTrpcRsp(httpResult);
      const result = httpResult.data;
      if (this.resultLog) {
        this.logger.info(`${funcName}|RESULT|CONTEXT: ${JSON.stringify(context)}|REQ: ${JSON.stringify(req.toObject ? e.toObject() : req)}|RESULT: ${JSON.stringify(result)}`);
      }
      return result;
    } catch (e) {
      this.logger.error(`${funcName}|ERROR|CONTEXT: ${JSON.stringify(context)}|REQ: ${e.stack}`);
      throw e;
    }
  }
}

module.exports = TrpcClient;
