const Hjson = require('hjson');
const TafConfigHelper = require('@tars/config');
const ConfigParser = require('@tars/utils').Config;

// 解析taf格式的配置文件
const parseCConfig = function (content) {
  const configParser = new ConfigParser();
  configParser.parseText(content, 'utf8');
  return configParser.data;
};

const formatters = {
  json: {
    parse: JSON.parse,
  },
  conf: {
    parse: parseCConfig,
  },
  c: {
    parse: parseCConfig,
  },
  hjson: {
    parse: Hjson.parse,
  },
  default: {
    parse: content => content,
  },
};

const tafConfigHelper = new TafConfigHelper();

const fileList = {};
const configs = {};

const addFormat = function (format, parseFunc) {
  formatters[format] = {
    parse: parseFunc,
  };
};

const extSplit = function (filename)  {
  filename = filename.replace('-', '_');
  const pos = filename.lastIndexOf('.');
  if (pos < 0) {
    return {
      name: filename,
      ext: '',
    };
  }

  return {
    name: filename.slice(0, pos),
    ext: filename.slice(pos + 1),
  };
};

const add = async function (filename, format) {
  const { name, ext } = extSplit(filename);
  format = format ? format : ext;
  fileList[filename] = {
    format,
  };
  const content = await tafConfigHelper.loadConfig(filename, { format: tafConfigHelper.FORMAT.TEXT });
  configs[name] = parse(content, format);
};

const parse = function (content, format) {
  let fmtter;
  if (format && formatters[format]) {
    fmtter = formatters[format];
  } else {
    fmtter = formatters.default;
  }
  return fmtter.parse(content);
};

process.on('message', (obj) => {
  if (obj.cmd === 'taf.loadconfig') {
    const DayLogger = require('./DayLog');
    DayLogger('TarsConfigLoader').info('loadconfig: ', JSON.stringify(obj));
    const filename = obj.data;
    add(filename);
  }
});

exports.addFormat = addFormat;
exports.add = add;
exports.configs = configs;
exports.tafConfigHelper = tafConfigHelper;
